<?php

namespace Mon\Product\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Mon\Product\Forms\Product as ProductForm;
use Mon\Product\Http\Requests\ProductRequest;
use Mon\Product\Interfaces\ProductRepositoryInterface;
use Mon\Product\Repositories\Eloquent\Filters\ProductResourceFilter;
use Mon\Product\Repositories\Eloquent\Presenters\ProductListPresenter;

/**
 * Resource controller class for product.
 */
class ProductResourceController extends BaseController
{

    /**
     * Initialize product resource controller.
     *
     *
     * @return null
     */
    public function __construct(ProductRepositoryInterface $product)
    {
        parent::__construct();
        $this->form = ProductForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('mon.product.modules'), 'product', guard_url('product'));
        $this->repository = $product;
    }

    /**
     * Display a list of product.
     *
     * @return Response
     */
    public function index(ProductRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(ProductResourceFilter::class)
            ->setPresenter(ProductListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('product::product.names'))
            ->view('product::product.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display product.
     *
     * @param Request $request
     * @param Model   $product
     *
     * @return Response
     */
    public function show(ProductRequest $request, ProductRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('product::product.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('product::product.show')
            ->output();
    }

    /**
     * Show the form for creating a new product.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(ProductRequest $request, ProductRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('product::product.name'))
            ->view('product::product.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new product.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ProductRequest $request, ProductRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('product::product.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('product/product/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/product/product'))
                ->redirect();
        }

    }

    /**
     * Show product for editing.
     *
     * @param Request $request
     * @param Model   $product
     *
     * @return Response
     */
    public function edit(ProductRequest $request, ProductRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('product::product.name'))
            ->view('product::product.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the product.
     *
     * @param Request $request
     * @param Model   $product
     *
     * @return Response
     */
    public function update(ProductRequest $request, ProductRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('product::product.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/product/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/product/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the product.
     *
     * @param Model   $product
     *
     * @return Response
     */
    public function destroy(ProductRequest $request, ProductRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('product::product.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/product/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/product/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
